@@ -51,7 +51,6 @@ struct GsdBackgroundManagerPrivate
 {
         GSettings   *settings;
         GnomeBG     *bg;
-        guint        timeout_id;
 
         GnomeBGCrossfade *fade;
 
@@ -70,6 +69,13 @@ G_DEFINE_TYPE (GsdBackgroundManager, gsd_background_manager, G_TYPE_OBJECT)
 
 static gpointer manager_object = NULL;
 
+static gboolean
+dont_draw_background (GsdBackgroundManager *manager)
+{
+        return !g_settings_get_boolean (manager->priv->settings,
+                                        "draw-background");
+}
+
 static gboolean
 nautilus_is_drawing_background (GsdBackgroundManager *manager)
 {
@@ -80,7 +86,6 @@ nautilus_is_drawing_background (GsdBackgroundManager *manager)
        unsigned long  nitems;
        unsigned long  bytes_after;
        unsigned char *data;
-       int            retval;
        Atom           wmclass_atom;
        gboolean       running;
        gint           error;
@@ -99,18 +104,18 @@ nautilus_is_drawing_background (GsdBackgroundManager *manager)
                return FALSE;
        }
 
-       retval = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-                                    GDK_ROOT_WINDOW (),
-                                    window_id_atom,
-                                    0,
-                                    1,
-                                    False,
-                                    XA_WINDOW,
-                                    &actual_type,
-                                    &actual_format,
-                                    &nitems,
-                                    &bytes_after,
-                                    &data);
+       XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                           GDK_ROOT_WINDOW (),
+                           window_id_atom,
+                           0,
+                           1,
+                           False,
+                           XA_WINDOW,
+                           &actual_type,
+                           &actual_format,
+                           &nitems,
+                           &bytes_after,
+                           &data);
 
        if (data != NULL) {
                nautilus_xid = *(Window *) data;
@@ -130,18 +135,18 @@ nautilus_is_drawing_background (GsdBackgroundManager *manager)
 
        gdk_error_trap_push ();
 
-       retval = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-                                    nautilus_xid,
-                                    wmclass_atom,
-                                    0,
-                                    24,
-                                    False,
-                                    XA_STRING,
-                                    &actual_type,
-                                    &actual_format,
-                                    &nitems,
-                                    &bytes_after,
-                                    &data);
+       XGetWindowProperty (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+                           nautilus_xid,
+                           wmclass_atom,
+                           0,
+                           24,
+                           False,
+                           XA_STRING,
+                           &actual_type,
+                           &actual_format,
+                           &nitems,
+                           &bytes_after,
+                           &data);
 
        error = gdk_error_trap_pop ();
 
@@ -184,7 +189,8 @@ draw_background (GsdBackgroundManager *manager,
         int         i;
 
 
-        if (nautilus_is_drawing_background (manager)) {
+        if (nautilus_is_drawing_background (manager) ||
+            dont_draw_background (manager)) {
                 return;
         }
 
@@ -292,34 +298,11 @@ setup_bg (GsdBackgroundManager *manager)
                                         manager->priv->settings);
 }
 
-static gboolean
-queue_draw_background (GsdBackgroundManager *manager)
+static void
+setup_bg_and_draw_background (GsdBackgroundManager *manager)
 {
-        manager->priv->timeout_id = 0;
-
         setup_bg (manager);
         draw_background (manager, FALSE);
-
-        return FALSE;
-}
-
-static void
-queue_timeout (GsdBackgroundManager *manager)
-{
-        if (manager->priv->timeout_id > 0)
-                return;
-
-        /* If the session finishes then check if nautilus is
-         * running and if not, set the background.
-         *
-         * We wait a few seconds after the session is up
-         * because nautilus tells the session manager that its
-         * ready before it sets the background.
-         */
-        manager->priv->timeout_id = g_timeout_add_seconds (8,
-                                                           (GSourceFunc)
-                                                           queue_draw_background,
-                                                           manager);
 }
 
 static void
@@ -342,7 +325,7 @@ on_session_manager_signal (GDBusProxy   *proxy,
         GsdBackgroundManager *manager = GSD_BACKGROUND_MANAGER (user_data);
 
         if (g_strcmp0 (signal_name, "SessionRunning") == 0) {
-                queue_timeout (manager);
+                setup_bg_and_draw_background (manager);
                 disconnect_session_manager_listener (manager);
         }
 }
@@ -420,6 +403,15 @@ connect_screen_signals (GsdBackgroundManager *manager)
         }
 }
 
+static void
+draw_background_changed (GSettings            *settings,
+                         const char           *key,
+                         GsdBackgroundManager *manager)
+{
+        if (dont_draw_background (manager) == FALSE)
+                setup_bg_and_draw_background (manager);
+}
+
 gboolean
 gsd_background_manager_start (GsdBackgroundManager *manager,
                               GError              **error)
@@ -430,6 +422,8 @@ gsd_background_manager_start (GsdBackgroundManager *manager,
         gnome_settings_profile_start (NULL);
 
         manager->priv->settings = g_settings_new ("org.gnome.desktop.background");
+        g_signal_connect (manager->priv->settings, "changed::draw-background",
+                          G_CALLBACK (draw_background_changed), manager);
 
         /* If this is set, nautilus will draw the background and is
 	 * almost definitely in our session.  however, it may not be
@@ -475,11 +469,6 @@ gsd_background_manager_stop (GsdBackgroundManager *manager)
                 p->settings = NULL;
         }
 
-        if (p->timeout_id != 0) {
-                g_source_remove (p->timeout_id);
-                p->timeout_id = 0;
-        }
-
         if (p->bg != NULL) {
                 g_object_unref (p->bg);
                 p->bg = NULL;
@@ -492,10 +481,6 @@ gsd_background_manager_set_property (GObject        *object,
                                      const GValue   *value,
                                      GParamSpec     *pspec)
 {
-        GsdBackgroundManager *self;
-
-        self = GSD_BACKGROUND_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -509,10 +494,6 @@ gsd_background_manager_get_property (GObject        *object,
                                      GValue         *value,
                                      GParamSpec     *pspec)
 {
-        GsdBackgroundManager *self;
-
-        self = GSD_BACKGROUND_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -526,9 +507,6 @@ gsd_background_manager_constructor (GType                  type,
                                     GObjectConstructParam *construct_properties)
 {
         GsdBackgroundManager      *background_manager;
-        GsdBackgroundManagerClass *klass;
-
-        klass = GSD_BACKGROUND_MANAGER_CLASS (g_type_class_peek (GSD_TYPE_BACKGROUND_MANAGER));
 
         background_manager = GSD_BACKGROUND_MANAGER (G_OBJECT_CLASS (gsd_background_manager_parent_class)->constructor (type,
                                                                                                                         n_construct_properties,
@@ -540,10 +518,6 @@ gsd_background_manager_constructor (GType                  type,
 static void
 gsd_background_manager_dispose (GObject *object)
 {
-        GsdBackgroundManager *background_manager;
-
-        background_manager = GSD_BACKGROUND_MANAGER (object);
-
         G_OBJECT_CLASS (gsd_background_manager_parent_class)->dispose (object);
 }
 
