@@ -58,6 +58,8 @@ struct GsdColorManagerPrivate
         GnomeRRScreen   *x11_screen;
         GHashTable      *edid_cache;
         GdkWindow       *gdk_window;
+        GnomeSettingsSessionState session_state;
+        GHashTable      *device_assign_hash;
 };
 
 enum {
@@ -493,12 +495,12 @@ out:
 static wchar_t *
 utf8_to_wchar_t (const char *src)
 {
-        gsize len;
-        gsize converted;
+        size_t len;
+        size_t converted;
         wchar_t *buf = NULL;
 
         len = mbstowcs (NULL, src, 0);
-        if (len < 0) {
+        if (len == (size_t) -1) {
                 g_warning ("Invalid UTF-8 in string %s", src);
                 goto out;
         }
@@ -734,6 +736,10 @@ gcm_session_generate_vcgt (CdProfile *profile, guint size)
         const gchar *filename;
         cmsHPROFILE lcms_profile = NULL;
 
+        /* invalid size */
+        if (size == 0)
+                goto out;
+
         /* not an actual profile */
         filename = cd_profile_get_filename (profile);
         if (filename == NULL)
@@ -774,6 +780,8 @@ gnome_rr_output_get_gamma_size (GnomeRROutput *output)
         gint len = 0;
 
         crtc = gnome_rr_output_get_crtc (output);
+        if (crtc == NULL)
+                return 0;
         gnome_rr_crtc_get_gamma (crtc,
                                  &len,
                                  NULL, NULL, NULL);
@@ -816,6 +824,15 @@ gcm_session_output_set_gamma (GnomeRROutput *output,
 
         /* send to LUT */
         crtc = gnome_rr_output_get_crtc (output);
+        if (crtc == NULL) {
+                ret = FALSE;
+                g_set_error (error,
+                             GSD_COLOR_MANAGER_ERROR,
+                             GSD_COLOR_MANAGER_ERROR_FAILED,
+                             "failed to get ctrc for %s",
+                             gnome_rr_output_get_name (output));
+                goto out;
+        }
         gnome_rr_crtc_set_gamma (crtc, array->len,
                                  red, green, blue);
 out:
@@ -831,11 +848,19 @@ gcm_session_device_set_gamma (GnomeRROutput *output,
                               GError **error)
 {
         gboolean ret = FALSE;
+        guint size;
         GPtrArray *clut = NULL;
 
         /* create a lookup table */
-        clut = gcm_session_generate_vcgt (profile,
-                                          gnome_rr_output_get_gamma_size (output));
+        size = gnome_rr_output_get_gamma_size (output);
+        if (size == 0) {
+                g_set_error_literal (error,
+                                     GSD_COLOR_MANAGER_ERROR,
+                                     GSD_COLOR_MANAGER_ERROR_FAILED,
+                                     "gamma size is zero");
+                goto out;
+        }
+        clut = gcm_session_generate_vcgt (profile, size);
         if (clut == NULL) {
                 g_set_error_literal (error,
                                      GSD_COLOR_MANAGER_ERROR,
@@ -869,6 +894,14 @@ gcm_session_device_reset_gamma (GnomeRROutput *output,
         g_debug ("falling back to dummy ramp");
         clut = g_ptr_array_new_with_free_func (g_free);
         size = gnome_rr_output_get_gamma_size (output);
+        if (size == 0) {
+                ret = FALSE;
+                g_set_error_literal (error,
+                                     GSD_COLOR_MANAGER_ERROR,
+                                     GSD_COLOR_MANAGER_ERROR_FAILED,
+                                     "gamma size is zero");
+                goto out;
+        }
         for (i = 0; i < size; i++) {
                 value = (i * 0xffff) / (size - 1);
                 data = g_new0 (GnomeRROutputClutItem, 1);
@@ -1018,6 +1051,10 @@ gcm_session_device_assign_connect_cb (GObject *object,
         GsdColorManager *manager = GSD_COLOR_MANAGER (user_data);
         GsdColorManagerPrivate *priv = manager->priv;
 
+        /* remove from assign array */
+        g_hash_table_remove (manager->priv->device_assign_hash,
+                             cd_device_get_object_path (device));
+
         /* get properties */
         ret = cd_device_connect_finish (device, res, &error);
         if (!ret) {
@@ -1128,6 +1165,19 @@ out:
 static void
 gcm_session_device_assign (GsdColorManager *manager, CdDevice *device)
 {
+        const gchar *key;
+        gpointer found;
+
+        /* are we already assigning this device */
+        key = cd_device_get_object_path (device);
+        found = g_hash_table_lookup (manager->priv->device_assign_hash, key);
+        if (found != NULL) {
+                g_debug ("assign for %s already in progress", key);
+                return;
+        }
+        g_hash_table_insert (manager->priv->device_assign_hash,
+                             g_strdup (key),
+                             GINT_TO_POINTER (TRUE));
         cd_device_connect (device,
                            NULL,
                            gcm_session_device_assign_connect_cb,
@@ -1347,6 +1397,69 @@ out:
                 g_ptr_array_unref (array);
 }
 
+static void
+gcm_session_profile_gamma_find_device_cb (GObject *object,
+                                          GAsyncResult *res,
+                                          gpointer user_data)
+{
+        CdClient *client = CD_CLIENT (object);
+        CdDevice *device = NULL;
+        GError *error = NULL;
+        GsdColorManager *manager = GSD_COLOR_MANAGER (user_data);
+
+        device = cd_client_find_device_by_property_finish (client,
+                                                           res,
+                                                           &error);
+        if (device == NULL) {
+                g_warning ("not found device %s: %s",
+                           cd_device_get_id (device),
+                           error->message);
+                g_error_free (error);
+                goto out;
+        }
+
+        /* get properties */
+        cd_device_connect (device,
+                           NULL,
+                           gcm_session_device_assign_connect_cb,
+                           manager);
+out:
+        if (device != NULL)
+                g_object_unref (device);
+}
+
+/* We have to reset the gamma tables each time as if the primary output
+ * has changed then different crtcs are going to be used.
+ * See https://bugzilla.gnome.org/show_bug.cgi?id=660164 for an example */
+static void
+gnome_rr_screen_output_changed_cb (GnomeRRScreen *screen,
+                                   GsdColorManager *manager)
+{
+        GnomeRROutput **outputs;
+        GsdColorManagerPrivate *priv = manager->priv;
+        guint i;
+
+        /* get X11 outputs */
+        outputs = gnome_rr_screen_list_outputs (priv->x11_screen);
+        if (outputs == NULL) {
+                g_warning ("failed to get outputs");
+                return;
+        }
+        for (i = 0; outputs[i] != NULL; i++) {
+                if (!gnome_rr_output_is_connected (outputs[i]))
+                        continue;
+
+                /* get CdDevice for this output */
+                cd_client_find_device_by_property (manager->priv->client,
+                                                   CD_DEVICE_METADATA_XRANDR_NAME,
+                                                   gnome_rr_output_get_name (outputs[i]),
+                                                   NULL,
+                                                   gcm_session_profile_gamma_find_device_cb,
+                                                   manager);
+        }
+
+}
+
 static void
 gcm_session_client_connect_cb (GObject *source_object,
                                GAsyncResult *res,
@@ -1406,6 +1519,9 @@ gcm_session_client_connect_cb (GObject *source_object,
         g_signal_connect (priv->x11_screen, "output-disconnected",
                           G_CALLBACK (gnome_rr_screen_output_removed_cb),
                           manager);
+        g_signal_connect (priv->x11_screen, "changed",
+                          G_CALLBACK (gnome_rr_screen_output_changed_cb),
+                          manager);
 
         g_signal_connect (priv->client, "profile-added",
                           G_CALLBACK (gcm_session_profile_added_assign_cb),
@@ -1950,21 +2066,29 @@ gcm_session_active_changed_cb (GnomeSettingsSession *session,
                                GParamSpec *pspec,
                                GsdColorManager *manager)
 {
-        GnomeSettingsSessionState state;
+        GnomeSettingsSessionState state_new;
+        GsdColorManagerPrivate *priv = manager->priv;
 
         /* not yet connected to the daemon */
-        if (!cd_client_get_connected (manager->priv->client))
+        if (!cd_client_get_connected (priv->client))
                 return;
 
-        /* when doing the fast-user-switch into a new account, load the
-         * new users chosen profiles */
-        state = gnome_settings_session_get_state (session);
-        if (state == GNOME_SETTINGS_SESSION_STATE_ACTIVE) {
-                g_debug ("Done switch to new account, reload devices");
+        /* When doing the fast-user-switch into a new account, load the
+         * new users chosen profiles.
+         *
+         * If this is the first time the GnomeSettingsSession has been
+         * loaded, then we'll get a change from unknown to active
+         * and we want to avoid reprobing the devices for that.
+         */
+        state_new = gnome_settings_session_get_state (session);
+        if (priv->session_state != GNOME_SETTINGS_SESSION_STATE_UNKNOWN &&
+            state_new == GNOME_SETTINGS_SESSION_STATE_ACTIVE) {
+                g_warning ("Done switch to new account, reload devices");
                 cd_client_get_devices (manager->priv->client, NULL,
                                        gcm_session_get_devices_cb,
                                        manager);
         }
+        priv->session_state = state_new;
 }
 
 static void
@@ -2020,6 +2144,7 @@ gsd_color_manager_init (GsdColorManager *manager)
 
         /* track the active session */
         priv->session = gnome_settings_session_new ();
+        priv->session_state = gnome_settings_session_get_state (priv->session);
         g_signal_connect (priv->session, "notify::state",
                           G_CALLBACK (gcm_session_active_changed_cb),
                           manager);
@@ -2033,6 +2158,12 @@ gsd_color_manager_init (GsdColorManager *manager)
                                                   g_free,
                                                   g_object_unref);
 
+        /* we don't want to assign devices multiple times at startup */
+        priv->device_assign_hash = g_hash_table_new_full (g_str_hash,
+                                                          g_str_equal,
+                                                          g_free,
+                                                          NULL);
+
         /* use DMI data for internal panels */
         priv->dmi = gcm_dmi_new ();
 
@@ -2076,6 +2207,7 @@ gsd_color_manager_finalize (GObject *object)
         g_object_unref (manager->priv->dmi);
         g_object_unref (manager->priv->session);
         g_hash_table_destroy (manager->priv->edid_cache);
+        g_hash_table_destroy (manager->priv->device_assign_hash);
         if (manager->priv->x11_screen != NULL)
                 g_object_unref (manager->priv->x11_screen);
 
