@@ -177,6 +177,20 @@ is_hyper (const gchar *string)
 	  (string[6] == '>'));
 }
 
+static inline gboolean
+is_primary (const gchar *string)
+{
+	return ((string[0] == '<') &&
+		(string[1] == 'p' || string[1] == 'P') &&
+		(string[2] == 'r' || string[2] == 'R') &&
+		(string[3] == 'i' || string[3] == 'I') &&
+		(string[4] == 'm' || string[4] == 'M') &&
+		(string[5] == 'a' || string[5] == 'A') &&
+		(string[6] == 'r' || string[6] == 'R') &&
+		(string[7] == 'y' || string[7] == 'Y') &&
+		(string[8] == '>'));
+}
+
 static inline gboolean
 is_keycode (const gchar *string)
 {
@@ -209,9 +223,9 @@ is_keycode (const gchar *string)
  * the virtual modifier represents the keyboard key, the concrete
  * modifier the actual Mod2-Mod5 bits in the key press event.
  *
- * Returns: %TRUE on success.
+ * Returns: %EGG_PARSE_ERROR_NONE on success.
  */
-gboolean
+EggParseError
 egg_accelerator_parse_virtual (const gchar            *accelerator,
                                guint                  *accelerator_key,
 			       guint                 **accelerator_codes,
@@ -220,7 +234,7 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
   guint keyval;
   GdkModifierType mods;
   gint len;
-  gboolean bad_keyval;
+  EggParseError ret;
 
   if (accelerator_key)
     *accelerator_key = 0;
@@ -231,7 +245,7 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
 
   g_return_val_if_fail (accelerator != NULL, FALSE);
 
-  bad_keyval = FALSE;
+  ret = EGG_PARSE_ERROR_NONE;
 
   keyval = 0;
   mods = 0;
@@ -312,6 +326,12 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
 	      len -= 7;
 	      mods |= EGG_VIRTUAL_SUPER_MASK;
 	    }
+	  else if (len >= 9 && is_primary (accelerator))
+	    {
+	      accelerator += 9;
+	      len -= 9;
+	      mods |= EGG_VIRTUAL_CONTROL_MASK;
+	    }
 	  else
 	    {
 	      gchar last_ch;
@@ -329,7 +349,7 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
 	{
           keyval = gdk_keyval_from_name (accelerator);
 
-          if (keyval == 0)
+          if (keyval == GDK_KEY_VoidSymbol)
 	    {
 	      /* If keyval is 0, then maybe it's a keycode.  Check for 0x## */
 	      if (len >= 4 && is_keycode (accelerator))
@@ -345,13 +365,13 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
 
 		  if (endptr == NULL || *endptr != '\000')
 		    {
-		      bad_keyval = TRUE;
+		      ret = EGG_PARSE_ERROR_INVALID_KEYCODE;
 		    }
 		  else if (accelerator_codes != NULL)
 		    {
 		      /* 0x00 is an invalid keycode too. */
 		      if (tmp_keycode == 0) {
-			bad_keyval = TRUE;
+			ret = EGG_PARSE_ERROR_INVALID_KEYCODE;
 		      } else {
 			*accelerator_codes = g_new0 (guint, 2);
 			(*accelerator_codes)[0] = tmp_keycode;
@@ -360,7 +380,7 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
 		}
 	      else
 		{
-		  bad_keyval = TRUE;
+		  ret = EGG_PARSE_ERROR_PARSE;
 		}
 	    }
 	  else if (accelerator_codes != NULL)
@@ -368,8 +388,8 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
 	      GdkKeymapKey *keys;
 	      gint n_keys, i, j;
 
-	      if (!gdk_keymap_get_entries_for_keyval (NULL, keyval, &keys, &n_keys)) {
- 	 	bad_keyval = TRUE;
+	      if (!gdk_keymap_get_entries_for_keyval (gdk_keymap_get_default (), keyval, &keys, &n_keys)) {
+	        ret = EGG_PARSE_ERROR_NOT_IN_KEYMAP;
 	      } else {
 		*accelerator_codes = g_new0 (guint, n_keys + 1);
 
@@ -381,7 +401,7 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
 		if (j == 0) {
 		  g_free (*accelerator_codes);
 		  *accelerator_codes = NULL;
-		  bad_keyval = TRUE;
+		  ret = EGG_PARSE_ERROR_NOT_IN_KEYMAP;
 	        }
 	        g_free (keys);
 	      }
@@ -397,7 +417,7 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
   if (accelerator_mods)
     *accelerator_mods = mods;
 
-  return !bad_keyval;
+  return ret;
 }
 
 /**
@@ -582,20 +602,20 @@ reload_modmap (GdkKeymap *keymap,
       mask = 0;
       for (j = 0; j < n_entries; ++j)
         {
-          if (keyvals[j] == GDK_Num_Lock)
+          if (keyvals[j] == GDK_KEY_Num_Lock)
             mask |= EGG_VIRTUAL_NUM_LOCK_MASK;
-          else if (keyvals[j] == GDK_Scroll_Lock)
+          else if (keyvals[j] == GDK_KEY_Scroll_Lock)
             mask |= EGG_VIRTUAL_SCROLL_LOCK_MASK;
-          else if (keyvals[j] == GDK_Meta_L ||
-                   keyvals[j] == GDK_Meta_R)
+          else if (keyvals[j] == GDK_KEY_Meta_L ||
+                   keyvals[j] == GDK_KEY_Meta_R)
             mask |= EGG_VIRTUAL_META_MASK;
-          else if (keyvals[j] == GDK_Hyper_L ||
-                   keyvals[j] == GDK_Hyper_R)
+          else if (keyvals[j] == GDK_KEY_Hyper_L ||
+                   keyvals[j] == GDK_KEY_Hyper_R)
             mask |= EGG_VIRTUAL_HYPER_MASK;
-          else if (keyvals[j] == GDK_Super_L ||
-                   keyvals[j] == GDK_Super_R)
+          else if (keyvals[j] == GDK_KEY_Super_L ||
+                   keyvals[j] == GDK_KEY_Super_R)
             mask |= EGG_VIRTUAL_SUPER_MASK;
-          else if (keyvals[j] == GDK_Mode_switch)
+          else if (keyvals[j] == GDK_KEY_Mode_switch)
             mask |= EGG_VIRTUAL_MODE_SWITCH_MASK;
         }
 
