@@ -37,10 +37,8 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
-#ifdef HAVE_X11_EXTENSIONS_XKB_H
 #include <X11/XKBlib.h>
 #include <X11/keysym.h>
-#endif
 
 #include "gnome-settings-profile.h"
 #include "gsd-keyboard-manager.h"
@@ -61,6 +59,7 @@
 #define KEY_INTERVAL       "repeat-interval"
 #define KEY_DELAY          "delay"
 #define KEY_CLICK_VOLUME   "click-volume"
+#define KEY_NUMLOCK_STATE  "numlock-state"
 
 #define KEY_BELL_VOLUME    "bell-volume"
 #define KEY_BELL_PITCH     "bell-pitch"
@@ -83,7 +82,6 @@ G_DEFINE_TYPE (GsdKeyboardManager, gsd_keyboard_manager, G_TYPE_OBJECT)
 
 static gpointer manager_object = NULL;
 
-#ifdef HAVE_X11_EXTENSIONS_XKB_H
 static gboolean
 xkb_set_keyboard_autorepeat_rate (guint delay, guint interval)
 {
@@ -92,43 +90,6 @@ xkb_set_keyboard_autorepeat_rate (guint delay, guint interval)
                                      delay,
                                      interval);
 }
-#endif
-
-static char *
-gsd_keyboard_get_hostname_key (void)
-{
-        /* FIXME disabled for now, as we need GSettingsList support:
-         * https://bugzilla.gnome.org/show_bug.cgi?id=622126 */
-#if 0
-        const char *hostname;
-
-        hostname = g_get_host_name ();
-
-        if (g_str_equal (hostname, "localhost") == FALSE &&
-            g_str_equal (hostname, "localhost.localdomain") == FALSE) {
-                char *escaped;
-                char *key;
-
-                /* FIXME, really escape? */
-                escaped = g_strdup (hostname);
-                key = g_strdup_printf ("host-%s-0-numlock-on", escaped);
-                g_free (escaped);
-                return key;
-        } else {
-                g_message ("NumLock remembering disabled because hostname is set to \"localhost\"");
-                return NULL;
-        }
-#endif
-        return NULL;
-}
-
-#ifdef HAVE_X11_EXTENSIONS_XKB_H
-
-typedef enum {
-        NUMLOCK_STATE_OFF = 0,
-        NUMLOCK_STATE_ON = 1,
-        NUMLOCK_STATE_UNKNOWN = 2
-} NumLockState;
 
 static void
 numlock_xkb_init (GsdKeyboardManager *manager)
@@ -166,50 +127,16 @@ numlock_NumLock_modifier_mask (void)
 }
 
 static void
-numlock_set_xkb_state (NumLockState new_state)
+numlock_set_xkb_state (GsdNumLockState new_state)
 {
         unsigned int num_mask;
         Display *dpy = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-        if (new_state != NUMLOCK_STATE_ON && new_state != NUMLOCK_STATE_OFF)
+        if (new_state != GSD_NUM_LOCK_STATE_ON && new_state != GSD_NUM_LOCK_STATE_OFF)
                 return;
         num_mask = numlock_NumLock_modifier_mask ();
         XkbLockModifiers (dpy, XkbUseCoreKbd, num_mask, new_state ? num_mask : 0);
 }
 
-static NumLockState
-numlock_get_gsettings_state (GSettings *settings)
-{
-        int   curr_state;
-        char *key;
-
-        key = gsd_keyboard_get_hostname_key ();
-        if (!key)
-                return NUMLOCK_STATE_UNKNOWN;
-
-        curr_state = g_settings_get_boolean (settings, key);
-
-        g_free (key);
-
-        return curr_state;
-}
-
-static void
-numlock_set_gsettings_state (GSettings   *settings,
-                             NumLockState new_state)
-{
-        char *key;
-
-        if (new_state != NUMLOCK_STATE_ON && new_state != NUMLOCK_STATE_OFF) {
-                return;
-        }
-
-        key = gsd_keyboard_get_hostname_key ();
-        if (key) {
-                g_settings_set_boolean (settings, key, new_state);
-                g_free (key);
-        }
-}
-
 static GdkFilterReturn
 numlock_xkb_callback (GdkXEvent *xev_,
                       GdkEvent  *gdkev_,
@@ -225,7 +152,7 @@ numlock_xkb_callback (GdkXEvent *xev_,
                         unsigned num_mask = numlock_NumLock_modifier_mask ();
                         unsigned locked_mods = xkbev->state.locked_mods;
                         int numlock_state = !! (num_mask & locked_mods);
-                        numlock_set_gsettings_state (manager->priv->settings, numlock_state);
+                        g_settings_set_enum (manager->priv->settings, KEY_NUMLOCK_STATE, numlock_state);
                 }
         }
         return GDK_FILTER_CONTINUE;
@@ -242,8 +169,6 @@ numlock_install_xkb_callback (GsdKeyboardManager *manager)
                                manager);
 }
 
-#endif /* HAVE_X11_EXTENSIONS_XKB_H */
-
 static guint
 _gsd_settings_get_uint (GSettings  *settings,
 			const char *key)
@@ -269,9 +194,7 @@ apply_settings (GSettings          *settings,
         int              bell_pitch;
         int              bell_duration;
         GsdBellMode      bell_mode;
-#ifdef HAVE_X11_EXTENSIONS_XKB_H
         gboolean         rnumlock;
-#endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
         repeat        = g_settings_get_boolean  (settings, KEY_REPEAT);
         click         = g_settings_get_boolean  (settings, KEY_CLICK);
@@ -284,9 +207,7 @@ apply_settings (GSettings          *settings,
         bell_mode = g_settings_get_enum (settings, KEY_BELL_MODE);
         bell_volume   = (bell_mode == GSD_BELL_MODE_ON) ? 50 : 0;
 
-#ifdef HAVE_X11_EXTENSIONS_XKB_H
         rnumlock      = g_settings_get_boolean  (settings, "remember-numlock-state");
-#endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
         gdk_error_trap_push ();
         if (repeat) {
@@ -294,9 +215,8 @@ apply_settings (GSettings          *settings,
 
                 XAutoRepeatOn (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
                 /* Use XKB in preference */
-#ifdef HAVE_X11_EXTENSIONS_XKB_H
                 rate_set = xkb_set_keyboard_autorepeat_rate (delay, interval);
-#endif
+
                 if (!rate_set)
                         g_warning ("Neither XKeyboard not Xfree86's keyboard extensions are available,\n"
                                    "no way to support keyboard autorepeat rate settings");
@@ -318,11 +238,9 @@ apply_settings (GSettings          *settings,
                                 KBKeyClickPercent | KBBellPercent | KBBellPitch | KBBellDuration,
                                 &kbdcontrol);
 
-#ifdef HAVE_X11_EXTENSIONS_XKB_H
         if (manager->priv->have_xkb && rnumlock) {
-                numlock_set_xkb_state (numlock_get_gsettings_state (manager->priv->settings));
+                numlock_set_xkb_state (g_settings_get_enum (manager->priv->settings, KEY_NUMLOCK_STATE));
         }
-#endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
         XSync (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), FALSE);
         gdk_error_trap_pop_ignored ();
@@ -347,9 +265,7 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
         /* Essential - xkb initialization should happen before */
         gsd_keyboard_xkb_init (manager);
 
-#ifdef HAVE_X11_EXTENSIONS_XKB_H
         numlock_xkb_init (manager);
-#endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
         /* apply current settings before we install the callback */
         gsd_keyboard_manager_apply_settings (manager);
@@ -357,9 +273,7 @@ start_keyboard_idle_cb (GsdKeyboardManager *manager)
         g_signal_connect (G_OBJECT (manager->priv->settings), "changed",
                           G_CALLBACK (apply_settings), manager);
 
-#ifdef HAVE_X11_EXTENSIONS_XKB_H
         numlock_install_xkb_callback (manager);
-#endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
         gnome_settings_profile_end (NULL);
 
@@ -393,13 +307,11 @@ gsd_keyboard_manager_stop (GsdKeyboardManager *manager)
                 p->settings = NULL;
         }
 
-#if HAVE_X11_EXTENSIONS_XKB_H
         if (p->have_xkb) {
                 gdk_window_remove_filter (NULL,
                                           numlock_xkb_callback,
                                           manager);
         }
-#endif /* HAVE_X11_EXTENSIONS_XKB_H */
 
         gsd_keyboard_xkb_shutdown ();
 }
