@@ -690,6 +690,7 @@ update_dialog (GsdMediaKeysManager *manager,
                                         CA_PROP_EVENT_ID, "audio-volume-change",
                                         CA_PROP_EVENT_DESCRIPTION, "volume changed through key press",
                                         CA_PROP_APPLICATION_ID, "org.gnome.VolumeControl",
+                                        CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
                                         NULL);
 }
 
@@ -2065,16 +2066,6 @@ on_bus_gotten (GObject             *source_object,
                           NULL,
                           (GAsyncReadyCallback) power_keyboard_ready_cb,
                           manager);
-
-        g_dbus_proxy_new (manager->priv->connection,
-                          G_DBUS_PROXY_FLAGS_NONE,
-                          NULL,
-                          "org.freedesktop.UPower",
-                          "/org/freedesktop/UPower",
-                          "org.freedesktop.UPower",
-                          NULL,
-                          (GAsyncReadyCallback) upower_ready_cb,
-                          manager);
 }
 
 static void
@@ -2088,6 +2079,16 @@ register_manager (GsdMediaKeysManager *manager)
                    manager->priv->bus_cancellable,
                    (GAsyncReadyCallback) on_bus_gotten,
                    manager);
+
+        g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
+                                  G_DBUS_PROXY_FLAGS_NONE,
+                                  NULL,
+                                  "org.freedesktop.UPower",
+                                  "/org/freedesktop/UPower",
+                                  "org.freedesktop.UPower",
+                                  NULL,
+                                  (GAsyncReadyCallback) upower_ready_cb,
+                                  manager);
 }
 
 GsdMediaKeysManager *
