@@ -48,6 +48,7 @@
 #define RENEW_INTERVAL        3500
 #define SUBSCRIPTION_DURATION 3600
 #define CONNECTING_TIMEOUT    60
+#define REASON_TIMEOUT        15000
 
 struct GsdPrintNotificationsManagerPrivate
 {
@@ -59,6 +60,8 @@ struct GsdPrintNotificationsManagerPrivate
         gboolean                      scp_handler_spawned;
         GPid                          scp_handler_pid;
         GList                        *timeouts;
+        GHashTable                   *printing_printers;
+        GList                        *active_notifications;
 };
 
 enum {
@@ -142,6 +145,15 @@ struct
         GsdPrintNotificationsManager *manager;
 } typedef TimeoutData;
 
+struct
+{
+        gchar *printer_name;
+        gchar *reason;
+        NotifyNotification *notification;
+        gulong notification_close_id;
+        GsdPrintNotificationsManager *manager;
+} typedef ReasonData;
+
 static void
 free_timeout_data (gpointer user_data)
 {
@@ -155,11 +167,46 @@ free_timeout_data (gpointer user_data)
         }
 }
 
+static void
+free_reason_data (gpointer user_data)
+{
+        ReasonData *data = (ReasonData *) user_data;
+
+        if (data) {
+                if (data->notification_close_id > 0 &&
+                    g_signal_handler_is_connected (data->notification,
+                                                   data->notification_close_id))
+                        g_signal_handler_disconnect (data->notification, data->notification_close_id);
+
+                g_object_unref (data->notification);
+
+                g_free (data->printer_name);
+                g_free (data->reason);
+
+                g_free (data);
+        }
+}
+
+static void
+notification_closed_cb (NotifyNotification *notification,
+                        gpointer            user_data)
+{
+        ReasonData *data = (ReasonData *) user_data;
+
+        if (data) {
+                data->manager->priv->active_notifications =
+                        g_list_remove (data->manager->priv->active_notifications, data);
+
+                free_reason_data (data);
+        }
+}
+
 static gboolean
 show_notification (gpointer user_data)
 {
         NotifyNotification *notification;
         TimeoutData        *data = (TimeoutData *) user_data;
+        ReasonData         *reason_data;
         GList              *tmp;
 
         if (!data)
@@ -171,11 +218,26 @@ show_notification (gpointer user_data)
 
         notify_notification_set_app_name (notification, _("Printers"));
         notify_notification_set_hint (notification,
-                                      "transient",
+                                      "resident",
                                       g_variant_new_boolean (TRUE));
-        notify_notification_show (notification, NULL);
+        notify_notification_set_timeout (notification, REASON_TIMEOUT);
 
-        g_object_unref (notification);
+        reason_data = g_new0 (ReasonData, 1);
+        reason_data->printer_name = g_strdup (data->printer_name);
+        reason_data->reason = g_strdup ("connecting-to-device");
+        reason_data->notification = notification;
+        reason_data->manager = data->manager;
+
+        reason_data->notification_close_id =
+                g_signal_connect (notification,
+                                  "closed",
+                                  G_CALLBACK (notification_closed_cb),
+                                  reason_data);
+
+        reason_data->manager->priv->active_notifications =
+                g_list_append (reason_data->manager->priv->active_notifications, reason_data);
+
+        notify_notification_show (notification, NULL);
 
         tmp = g_list_find (data->manager->priv->timeouts, data);
         if (tmp) {
@@ -198,7 +260,6 @@ on_cups_notification (GDBusConnection *connection,
         GsdPrintNotificationsManager *manager = (GsdPrintNotificationsManager *) user_data;
         gboolean                     printer_is_accepting_jobs;
         gboolean                     my_job = FALSE;
-        gboolean                     connecting;
         http_t                      *http;
         gchar                       *printer_name = NULL;
         gchar                       *display_name = NULL;
@@ -238,7 +299,7 @@ on_cups_notification (GDBusConnection *connection,
                 N_("Not connected?"),
                 /* Translators: One or more covers on the printer are open (same as in system-config-printer) */
                 N_("Cover open"),
-                /* Trnaslators: A filter or backend is not installed (same as in system-config-printer) */
+                /* Translators: A filter or backend is not installed (same as in system-config-printer) */
                 N_("Printer configuration error"),
                 /* Translators: One or more doors on the printer are open (same as in system-config-printer) */
                 N_("Door open"),
@@ -264,7 +325,7 @@ on_cups_notification (GDBusConnection *connection,
                 N_("Printer '%s' may not be connected."),
                 /* Translators: One or more covers on the printer are open (same as in system-config-printer) */
                 N_("The cover is open on printer '%s'."),
-                /* Trnaslators: A filter or backend is not installed (same as in system-config-printer) */
+                /* Translators: A filter or backend is not installed (same as in system-config-printer) */
                 N_("There is a missing print filter for "
                    "printer '%s'."),
                 /* Translators: One or more doors on the printer are open (same as in system-config-printer) */
@@ -323,11 +384,14 @@ on_cups_notification (GDBusConnection *connection,
                 else {
                         job_uri = g_strdup_printf ("ipp://localhost/jobs/%d", job_id);
 
-                        request = ippNewRequest(IPP_GET_JOB_ATTRIBUTES);
-                        ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "job-uri", NULL, job_uri);
-                        ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
+                        request = ippNewRequest (IPP_GET_JOB_ATTRIBUTES);
+                        ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_URI,
+                                      "job-uri", NULL, job_uri);
+                        ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+                                     "requesting-user-name", NULL, cupsUser ());
+                        ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
                                      "requested-attributes", NULL, "job-originating-user-name");
-                        response = cupsDoRequest(http, request, "/");
+                        response = cupsDoRequest (http, request, "/");
 
                         if (response) {
                                 if (response->request.status.status_code <= IPP_OK_CONFLICT &&
@@ -365,6 +429,9 @@ on_cups_notification (GDBusConnection *connection,
                 cupsFreeDests (manager->priv->num_dests, manager->priv->dests);
                 manager->priv->num_dests = cupsGetDests (&manager->priv->dests);
         } else if (g_strcmp0 (signal_name, "JobCompleted") == 0 && my_job) {
+                g_hash_table_remove (manager->priv->printing_printers,
+                                     printer_name);
+
                 /* FIXME: get a better human readable name */
                 display_name = g_strdup (printer_name);
 
@@ -406,11 +473,21 @@ on_cups_notification (GDBusConnection *connection,
 
                 switch (job_state) {
                         case IPP_JOB_PROCESSING:
+                                g_hash_table_insert (manager->priv->printing_printers,
+                                                     g_strdup (printer_name), NULL);
+
                                 /* Translators: A job is printing */
                                 primary_text = g_strdup (_("Printing"));
                                 /* Translators: "print-job xy" on a printer */
                                 secondary_text = g_strdup_printf (_("\"%s\" on %s"), job_name, display_name);
                                 break;
+                        case IPP_JOB_STOPPED:
+                        case IPP_JOB_CANCELED:
+                        case IPP_JOB_ABORTED:
+                        case IPP_JOB_COMPLETED:
+                                g_hash_table_remove (manager->priv->printing_printers,
+                                                     printer_name);
+                                break;
                         default:
                                 break;
                 }
@@ -421,6 +498,9 @@ on_cups_notification (GDBusConnection *connection,
                                               manager->priv->num_dests);
 
                 if (job_state == IPP_JOB_PROCESSING) {
+                        g_hash_table_insert (manager->priv->printing_printers,
+                                             g_strdup (printer_name), NULL);
+
                         /* Translators: A job is printing */
                         primary_text = g_strdup (_("Printing"));
                         /* Translators: "print-job xy" on a printer */
@@ -431,84 +511,17 @@ on_cups_notification (GDBusConnection *connection,
                 const gchar  *tmp_printer_state_reasons = NULL;
                 GSList       *added_reasons = NULL;
                 GSList       *tmp_list = NULL;
+                GList        *tmp;
                 gchar       **old_state_reasons = NULL;
                 gchar       **new_state_reasons = NULL;
                 gint          i, j;
 
-                /* FIXME: get a better human readable name */
-                display_name = g_strdup (printer_name);
-
-                dest = cupsGetDest (printer_name,
-                                    NULL,
-                                    manager->priv->num_dests,
-                                    manager->priv->dests);
-                if (dest)
-                        tmp_printer_state_reasons = cupsGetOption ("printer-state-reasons",
-                                                                   dest->num_options,
-                                                                   dest->options);
-
-                if (tmp_printer_state_reasons)
-                        old_state_reasons = g_strsplit (tmp_printer_state_reasons, ",", -1);
-
-                cupsFreeDests (manager->priv->num_dests, manager->priv->dests);
-                manager->priv->num_dests = cupsGetDests (&manager->priv->dests);
-
-                dest = cupsGetDest (printer_name,
-                                    NULL,
-                                    manager->priv->num_dests,
-                                    manager->priv->dests);
-                if (dest)
-                        tmp_printer_state_reasons = cupsGetOption ("printer-state-reasons",
-                                                                   dest->num_options,
-                                                                   dest->options);
-
-                if (tmp_printer_state_reasons)
-                        new_state_reasons = g_strsplit (tmp_printer_state_reasons, ",", -1);
-
-                if (new_state_reasons)
-                        qsort (new_state_reasons,
-                               g_strv_length (new_state_reasons),
-                               sizeof (gchar *),
-                               strcmp0);
-
-                if (old_state_reasons) {
-                        qsort (old_state_reasons,
-                               g_strv_length (old_state_reasons),
-                               sizeof (gchar *),
-                               strcmp0);
-
-                        j = 0;
-                        for (i = 0; i < g_strv_length (new_state_reasons); i++) {
-                                while (old_state_reasons[j] &&
-                                       g_strcmp0 (old_state_reasons[j], new_state_reasons[i]) < 0)
-                                        j++;
-
-                                if (old_state_reasons[j] == NULL ||
-                                    g_strcmp0 (old_state_reasons[j], new_state_reasons[i]) != 0)
-                                        added_reasons = g_slist_append (added_reasons,
-                                                                        new_state_reasons[i]);
-                        }
-                }
-                else {
-                        for (i = 0; i < g_strv_length (new_state_reasons); i++) {
-                                added_reasons = g_slist_append (added_reasons,
-                                                                new_state_reasons[i]);
-                        }
-                }
-
-                connecting = FALSE;
-                if (new_state_reasons) {
-                        for (i = 0; i < g_strv_length (new_state_reasons); i++) {
-                                if (g_strcmp0 (new_state_reasons[i], "connecting-to-device") == 0) {
-                                        connecting = TRUE;
-                                        break;
-                                }
-                        }
-                }
-
-                if (!connecting) {
+                /* Remove timeout which shows notification about possible disconnection of printer
+                 * if "connecting-to-device" has vanished.
+                 */
+                if (printer_state_reasons == NULL ||
+                    g_strrstr (printer_state_reasons, "connecting-to-device") == NULL) {
                         TimeoutData *data;
-                        GList       *tmp;
 
                         for (tmp = manager->priv->timeouts; tmp; tmp = g_list_next (tmp)) {
                                 data = (TimeoutData *) tmp->data;
@@ -521,45 +534,159 @@ on_cups_notification (GDBusConnection *connection,
                         }
                 }
 
-                for (tmp_list = added_reasons; tmp_list; tmp_list = tmp_list->next) {
-                        gchar *data = (gchar *) tmp_list->data;
-                        for (j = 0; j < G_N_ELEMENTS (reasons); j++) {
-                                if (strncmp (data,
-                                             reasons[j],
-                                             strlen (reasons[j])) == 0) {
-                                        NotifyNotification *notification;
+                for (tmp = manager->priv->active_notifications; tmp; tmp = g_list_next (tmp)) {
+                        ReasonData *reason_data = (ReasonData *) tmp->data;
+                        GList      *remove_list;
 
-                                        if (g_strcmp0 (reasons[j], "connecting-to-device") == 0) {
-                                                TimeoutData *data;
+                        if (printer_state_reasons == NULL ||
+                            (g_strcmp0 (printer_name, reason_data->printer_name) == 0 &&
+                             g_strrstr (printer_state_reasons, reason_data->reason) == NULL)) {
 
-                                                data = g_new0 (TimeoutData, 1);
-                                                data->printer_name = g_strdup (printer_name);
-                                                data->primary_text = g_strdup (statuses_first[j]);
-                                                data->secondary_text = g_strdup_printf (statuses_second[j], printer_name);
-                                                data->manager = manager;
+                                if (reason_data->notification_close_id > 0 &&
+                                    g_signal_handler_is_connected (reason_data->notification,
+                                                                   reason_data->notification_close_id)) {
+                                        g_signal_handler_disconnect (reason_data->notification,
+                                                                     reason_data->notification_close_id);
+                                        reason_data->notification_close_id = 0;
+                                }
 
-                                                data->timeout_id = g_timeout_add_seconds (CONNECTING_TIMEOUT, show_notification, data);
-                                                manager->priv->timeouts = g_list_append (manager->priv->timeouts, data);
-                                        }
-                                        else {
-                                                gchar *second_row = g_strdup_printf (statuses_second[j], printer_name);
+                                notify_notification_close (reason_data->notification, NULL);
 
-                                                notification = notify_notification_new (statuses_first[j],
-                                                                                        second_row,
-                                                                                        "printer-symbolic");
-                                                notify_notification_set_app_name (notification, _("Printers"));
-                                                notify_notification_set_hint (notification,
-                                                                              "transient",
-                                                                              g_variant_new_boolean (TRUE));
-                                                notify_notification_show (notification, NULL);
+                                remove_list = tmp;
+                                tmp = g_list_next (tmp);
+                                manager->priv->active_notifications =
+                                        g_list_remove_link (manager->priv->active_notifications, remove_list);
 
-                                                g_object_unref (notification);
-                                                g_free (second_row);
+                                g_list_free_full (remove_list, free_reason_data);
+                        }
+                }
+
+                /* Check whether we are printing on this printer right now. */
+                if (g_hash_table_lookup_extended (manager->priv->printing_printers, printer_name, NULL, NULL)) {
+                        /* FIXME: get a better human readable name */
+                        display_name = g_strdup (printer_name);
+
+                        dest = cupsGetDest (printer_name,
+                                            NULL,
+                                            manager->priv->num_dests,
+                                            manager->priv->dests);
+                        if (dest)
+                                tmp_printer_state_reasons = cupsGetOption ("printer-state-reasons",
+                                                                           dest->num_options,
+                                                                           dest->options);
+
+                        if (tmp_printer_state_reasons)
+                                old_state_reasons = g_strsplit (tmp_printer_state_reasons, ",", -1);
+
+                        cupsFreeDests (manager->priv->num_dests, manager->priv->dests);
+                        manager->priv->num_dests = cupsGetDests (&manager->priv->dests);
+
+                        dest = cupsGetDest (printer_name,
+                                            NULL,
+                                            manager->priv->num_dests,
+                                            manager->priv->dests);
+                        if (dest)
+                                tmp_printer_state_reasons = cupsGetOption ("printer-state-reasons",
+                                                                           dest->num_options,
+                                                                           dest->options);
+
+                        if (tmp_printer_state_reasons)
+                                new_state_reasons = g_strsplit (tmp_printer_state_reasons, ",", -1);
+
+                        if (new_state_reasons)
+                                qsort (new_state_reasons,
+                                       g_strv_length (new_state_reasons),
+                                       sizeof (gchar *),
+                                       strcmp0);
+
+                        if (old_state_reasons) {
+                                qsort (old_state_reasons,
+                                       g_strv_length (old_state_reasons),
+                                       sizeof (gchar *),
+                                       strcmp0);
+
+                                j = 0;
+                                for (i = 0; new_state_reasons && i < g_strv_length (new_state_reasons); i++) {
+                                        while (old_state_reasons[j] &&
+                                               g_strcmp0 (old_state_reasons[j], new_state_reasons[i]) < 0)
+                                                j++;
+
+                                        if (old_state_reasons[j] == NULL ||
+                                            g_strcmp0 (old_state_reasons[j], new_state_reasons[i]) != 0)
+                                                added_reasons = g_slist_append (added_reasons,
+                                                                                new_state_reasons[i]);
+                                }
+                        }
+                        else {
+                                for (i = 0; new_state_reasons && i < g_strv_length (new_state_reasons); i++) {
+                                        added_reasons = g_slist_append (added_reasons,
+                                                                        new_state_reasons[i]);
+                                }
+                        }
+
+                        for (tmp_list = added_reasons; tmp_list; tmp_list = tmp_list->next) {
+                                gchar *data = (gchar *) tmp_list->data;
+                                for (j = 0; j < G_N_ELEMENTS (reasons); j++) {
+                                        if (strncmp (data,
+                                                     reasons[j],
+                                                     strlen (reasons[j])) == 0) {
+                                                NotifyNotification *notification;
+
+                                                if (g_strcmp0 (reasons[j], "connecting-to-device") == 0) {
+                                                        TimeoutData *data;
+
+                                                        data = g_new0 (TimeoutData, 1);
+                                                        data->printer_name = g_strdup (printer_name);
+                                                        data->primary_text = g_strdup (statuses_first[j]);
+                                                        data->secondary_text = g_strdup_printf (statuses_second[j], printer_name);
+                                                        data->manager = manager;
+
+                                                        data->timeout_id = g_timeout_add_seconds (CONNECTING_TIMEOUT, show_notification, data);
+                                                        manager->priv->timeouts = g_list_append (manager->priv->timeouts, data);
+                                                }
+                                                else {
+                                                        ReasonData *reason_data;
+                                                        gchar *second_row = g_strdup_printf (statuses_second[j], printer_name);
+
+                                                        notification = notify_notification_new (statuses_first[j],
+                                                                                                second_row,
+                                                                                                "printer-symbolic");
+                                                        notify_notification_set_app_name (notification, _("Printers"));
+                                                        notify_notification_set_hint (notification,
+                                                                                      "resident",
+                                                                                      g_variant_new_boolean (TRUE));
+                                                        notify_notification_set_timeout (notification, REASON_TIMEOUT);
+
+                                                        reason_data = g_new0 (ReasonData, 1);
+                                                        reason_data->printer_name = g_strdup (printer_name);
+                                                        reason_data->reason = g_strdup (reasons[j]);
+                                                        reason_data->notification = notification;
+                                                        reason_data->manager = manager;
+
+                                                        reason_data->notification_close_id =
+                                                                g_signal_connect (notification,
+                                                                                  "closed",
+                                                                                  G_CALLBACK (notification_closed_cb),
+                                                                                  reason_data);
+
+                                                        manager->priv->active_notifications =
+                                                                g_list_append (manager->priv->active_notifications, reason_data);
+
+                                                        notify_notification_show (notification, NULL);
+
+                                                        g_free (second_row);
+                                                }
                                         }
                                 }
                         }
+                        g_slist_free (added_reasons);
                 }
-                g_slist_free (added_reasons);
+
+                if (new_state_reasons)
+                        g_strfreev (new_state_reasons);
+
+                if (old_state_reasons)
+                        g_strfreev (old_state_reasons);
         }
 
         g_free (display_name);
@@ -715,6 +842,8 @@ gsd_print_notifications_manager_start (GsdPrintNotificationsManager *manager,
         manager->priv->num_dests = 0;
         manager->priv->scp_handler_spawned = FALSE;
         manager->priv->timeouts = NULL;
+        manager->priv->printing_printers = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+        manager->priv->active_notifications = NULL;
 
         renew_subscription (manager);
         g_timeout_add_seconds (RENEW_INTERVAL, renew_subscription, manager);
@@ -760,6 +889,7 @@ void
 gsd_print_notifications_manager_stop (GsdPrintNotificationsManager *manager)
 {
         TimeoutData *data;
+        ReasonData  *reason_data;
         GList       *tmp;
 
         g_debug ("Stopping print-notifications manager");
@@ -771,6 +901,8 @@ gsd_print_notifications_manager_stop (GsdPrintNotificationsManager *manager)
         if (manager->priv->subscription_id >= 0)
                 cancel_subscription (manager->priv->subscription_id);
 
+        g_hash_table_destroy (manager->priv->printing_printers);
+
         manager->priv->cups_bus_connection = NULL;
 
         if (manager->priv->cups_proxy != NULL) {
@@ -785,6 +917,22 @@ gsd_print_notifications_manager_stop (GsdPrintNotificationsManager *manager)
         }
         g_list_free_full (manager->priv->timeouts, free_timeout_data);
 
+        for (tmp = manager->priv->active_notifications; tmp; tmp = g_list_next (tmp)) {
+                reason_data = (ReasonData *) tmp->data;
+                if (reason_data) {
+                        if (reason_data->notification_close_id > 0 &&
+                            g_signal_handler_is_connected (reason_data->notification,
+                                                           reason_data->notification_close_id)) {
+                                g_signal_handler_disconnect (reason_data->notification,
+                                                             reason_data->notification_close_id);
+                                reason_data->notification_close_id = 0;
+                        }
+
+                        notify_notification_close (reason_data->notification, NULL);
+                }
+        }
+        g_list_free_full (manager->priv->active_notifications, free_reason_data);
+
         scp_handler (manager, FALSE);
 }
 
